// use std::net::SocketAddr;

use std::{collections::HashMap, error::Error, fmt::Display};

use super::{
    methods::Method,
    routes::{Data, Uri},
};

type HeaderMap = Vec<String>;
#[derive(Clone)]
pub struct Request<'a> {
    pub uri: Uri<'a>,
    pub headers: HeaderMap,
    pub method: Method,
    // pub connection: ConnectionMeta,
}

// struct ConnectionMeta {
//     remote: Option<SocketAddr>,
//     // certificates
// }

#[derive(Clone, Debug, Copy, PartialEq, Eq, PartialOrd, Ord)]
pub enum MediaType {
    Json,
    Plain,
    Html,
}

#[derive(Debug)]
pub enum ParseErrors {
    NoData,
    BadData,
}

#[derive(Debug)]
pub struct ParseFormError {
    pub error: ParseErrors,
}

impl Display for ParseFormError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", self.error)
    }
}

impl Display for ParseErrors {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            ParseErrors::NoData => write!(f, "No Data at key"),
            ParseErrors::BadData => write!(f, "Bad Data at key"),
        }
    }
}

impl Error for ParseFormError {}

impl Request<'_> {
    pub fn can_have_body(&self) -> bool {
        match self.method {
            Method::Post | Method::Put | Method::Patch | Method::Delete => true,
            _ => false,
        }
    }
    // pub fn get_post_form_key<T: FromRequest>(&self, data: Data) -> T {}
    pub fn get_get_form_keys(
        &self,
        keys: Vec<&str>,
    ) -> Result<HashMap<&str, &str>, ParseFormError> {
        let data = if let Some(val) = self.uri.split_once("?") {
            val
        } else {
            return Err(ParseFormError {
                error: ParseErrors::NoData,
            });
        };
        let data = data
            .1
            .split("&")
            .map(|kvp| kvp.split_once("="))
            .collect::<Vec<Option<(&str, &str)>>>();

        let mut values: HashMap<&str, &str> = HashMap::new();
        for kvp in data {
            let kvp = if let Some(kvp) = kvp {
                kvp
            } else {
                continue;
            };
            values.insert(kvp.0, kvp.1);
        }
        let mut response = HashMap::new();
        for key in keys {
            let entry = if let Some(val) = values.get_key_value(key) {
                val
            } else {
                return Err(ParseFormError {
                    error: ParseErrors::NoData,
                });
            };
            response.insert(*entry.0, *entry.1);
        }
        Ok(response)
    }
    pub fn get_form_keys_with_data(
        &self,
        _keys: Vec<&str>,
        _data_buffer: Data,
    ) -> Result<HashMap<&str, &str>, ParseFormError> {
        todo!()
    }
}
